#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'wenchao.hao'
__version__ = '0.1.0'

"""
warp redis-py 3.0.0(see https://pypi.org/project/redis/)    
        & redis-py-cluster 2.0.0 (see https://pypi.org/project/redis-py-cluster/)
"""

from ._redis import Factory as RedisFactory, Client