#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'wenchao.hao'
__version__ = '0.1.0'

"""
config package. for read yaml、yml、json file.
    像springboot一样读取yaml文件装载到对象options中(but python的dict天生支持json, haha)
"""

from ._config import *