#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'wenchao.hao'
__version__ = '0.1.0'

"""
warp pymongo 3.9.0(see https://pypi.org/project/pymongo/)    
"""

from ._mongo import Factory as MongoFactory, Client